    /*  Definitions of major parameter structures */

#define MAXFREQ   16
#define MAXINDEX 256
#define LAG 0
#define GLOBAL 1

#include "plot_struct.h"

struct type_param 
    {
                                        /* Filled in by time_range() */
    double      start_nom;              /* nom. scan start time from ovex (sBOY) */
    double      stop_nom;               /*  "     "  stop   "     "    "  (sBOY) */
    int         start_offset;
    int         stop_offset;
    double      start;                  /* actual start time of earliest AP (sBOY) */
    double      stop;                   /*    "   stop    "   " latest    " (sBOY) */
    double      reftime;                /* fourfit ref. time (midpoint in s since BOY) */
    double      frt_offset;             /* Offset of ref time from scan start */
    double      acc_period;             /* Length of accumulation period (sec)  */
    int         minap;                  /* First AP number */
    int         maxap;                  /* Last AP number */
    int         num_ap;                 /* Number of Accumulation Periods */
    char        baseline[2];            /* Carried for convenience */
    short       corr_type;              /* Determines type_120 format */
    short       pol;                    /* Polarization type (LL,RR,LR,RL=0,1,2,3) */
    char        *rf_fglist;             /* List of fgroups when refringing */
    double      cor_limit;              /* Maximum correlation amplitude */
    int         bits_sample;            /* Bits per sample (1 or 2) */
    double      samp_period;            /* Length of sample period */
    short       pc_mode[2];             /* Phase cal mode by station */
    double      win_sb[2];              /* Single band search window microsec */
    double      win_dr[2];              /* Delay rate search window microsec per sec */
    double      win_mb[2];              /* Multi band search window  microsec */
    double      inv_sigma;              /* 1 / (1 sigma noise) */
    double      epoch_time;             /* Epoch time */
    int         cormode;                /* determines type 120 union */
    int         nlags;                  /* Variable in Mk4 */
    double      ref_freq;               /* Reference frequency in MHz */
    double      ah_tref;                /* Ad hoc reference time (s past hour) */
    double      ah_period;              /* Ad hoc sinusoid period (s) */
    double      ah_amp;                 /* Ad hoc sinusoid amplitude (rad) */
    double      ah_poly[6];             /* Ad hoc polynomial coefficients */
    int         ah_phase;               /* Ad hoc phase correction mode */
    unsigned long bocf_period;          /* Correlator frame in systicks */
    int         bocfs_per_ap;           /* Correlator frames per AP */
    short       su_number[2];           /* Ref and remote SU number */
    };

#define WIN_EDGE_SBD   0x01             /* masks for status.interp_err */
#define WIN_EDGE_MBD   0x02
#define WIN_EDGE_RATE  0x04
#define INTP_ERR_SBD   0x08
#define INTP_ERR_MBD   0x10
#define INTP_ERR_RATE  0x20


struct type_status 
    {
    double      freq_space;             /* Freq spacing in FFT to MBdelay */
    int         grid_points;            /* # of points in FFT to MBdelay */
    double      freq_spread;            /* Sum of (Freq - avg freq) squared */
    int         large_errors;           /* # of large errors */
    int         sliver_errors;          /* # of AP's with sliver (too little data) errors */
    int         zero_errors;            /* # of AP's with zero (some lag count 0) errors */
    int         total_ap;               /* Total # of ap's processed (both sb's )*/
    float       total_ap_frac;          /* Same, but with microediting */
    int         ap_num[2][MAXFREQ];     /* # of aps for each sideband & freq */
    float       ap_frac[2][MAXFREQ];    /* Same, but with microediting */
    double      epoch_err[MAXFREQ];     /* Epoch error (offset from ref. time) */
    double      epoch_off_cent;         /* Epoch offset from center  */
    int         mb_index[MAXFREQ];      /* index for freq in FFT to MBdelay */
    double      pc_meas[MAXFREQ][2];    /* phase-cal phase as measured */
    double      pc_phase[MAXFREQ][2];   /* phase-cal phase as used */
    double      pc_amp[MAXFREQ][2];     /* phase-cal magnitude */
    double      pc_offset[MAXFREQ][2];  /* Additive/manual phasecal values */
    double      pcals_accum[2];         /* # of phase-cals for current channel */
    double      lsb_phoff[2];           /* LSB phase offset in radians */
    int         space_err;              /* =1 if error if freq spacing is too large */
    int         drsp_size;              /* # of points in delay rate spectrum */
    int         f_rate_size;            /* # of points in FFT to Fringe rate */
    int         freq;                   /* Current frequency */
    int         lag;                    /* Current lag # */
    int         dr;                     /* Current delay rate # */
    int         mbd;                    /* Current multi band delay # */
    int         ap;                     /* Current acc per */
    double      dr_max[2*MAXLAG];       /* Delay rate at max by lag */
    double      dr_max_global;          /* Delay rate at global max */
    double      corr_dr_max;            /* Delay rate max corrected for pcal rate */
    double      sbd_max;                /* single band delay at max */
    double      mbd_max[2*MAXLAG];      /* multi band delay at max by lag */
    double      mbd_max_global;         /* multi band delay at global max */
    int         max_delchan;            /* Single band delay channel at max */
    double      delres_max;             /* max value of delay resolution function */
    double      rate_sep;               /* Rate separation of FFT points for freq #1 */
    double      mbd_sep;                /* Multi band delay separation of FFt points */
    double      sbd_sep;                /* Single band delay separation  microsec */
    double      search_amp;             /* Cor. amplitude after initial search */
    double      interp_amp;             /* Amplitude after 1st interpolation */
    double      pc_rate[2];             /* Interpolated phasecal rate */
    double      int_pc_amp[2];          /* Interpolated phasecal amplitude */
    double      amp_corr_fact;          /* Amplitude corr. factor for interpolation */
    double      amp_rate_corr;          /* Rate correction to amplitude */
    int         interp_err;             /* =1 if error in interpolation */
    double      AN;                     /* Normalization from FFT (from FRNGE) */
    int         win_sb[2];              /* Single band search indices */
    int         win_dr[2];              /* Delay rate search indices */
    int         win_mb[2];              /* Multi band search indices */
    int         pts_searched;           /* Number of points searched */
    complex     fringe[MAXFREQ+1];      /* Fringe phase & amp for each freq */
    double      sbdbox[MAXFREQ+1];      /* Single band delay box for each freq */
    double      snr;                    /* Signal to Noise ratio */
    double      prob_false;             /* Probability of false detection */
    int         nseg;                   /* Number of segments in computing time RMS */
    int         apseg;                  /* Number of APs in each segment */
    double      th_timerms_phase;       /* theoretical RMS phase with time */
    double      th_timerms_amp;         /* theoretical RMS amp with time */
    double      th_freqrms_phase;       /* theoretical RMS phase with freq */
    double      th_freqrms_amp;         /* theoretical RMS amp with freq */
    double      timerms_phase;          /* RMS phase with time */
    double      timerms_amp;            /* RMS amp with time */
    double      freqrms_phase;          /* RMS phase with freq */
    double      freqrms_amp;            /* RMS amp with freq */
    double      integ_time;             /* Total integration time */
    double      inc_avg_phase;          /* Fringe phase for incoh. avg. over freq */
    double      inc_avg_amp;            /* Fringe amp. from inc. sum   of all points */
    double      coh_avg_phase;          /* Fringe phase from coh. sum over freq */
    double      inc_avg_amp_freq;       /* Fringe amp from incoh sum over freq */
    double      GHA;                    /* Greenwich hour angle (hrs) */
    double      apdelay;                /* Adjusted apriori delay */
    double      amp_err;                /* Error in fringe amplitude */
    double      apphase;                /* Apriori phase */
    double      resid_phase;            /* Residual phase */
    double      mod_resid_phase;        /* Modified residual phase */
    double      phase_err;              /* Error in fringe phase */
    double      resid_ph_delay;         /* Residual phase delay */
    double      ph_delay_err;           /* Error in phase delay */
    double      sbavg;
    double      rate_ra_width;
    double      rate_dec_width;
    double      sbd_ra_width;
    double      sbd_dec_width;
    };
