/*#include <sys/types.h>*/
#include <sys/stat.h>
#include "math.h"
#include "vex.h"                        /* Needed for the VEX root file format */
#include "mk4_data.h"                   /* Definitions of Mk4 data structures */
#include "control.h"                    /* Definition of control structure */
#include "pass_struct.h"                /* Pass-specific parameters */
#include "fstruct.h"
/*#include "ccfile.h"*/
#define MAXFREQ 16   /*16 standard, but may be deviant settings needing 32 or 64..*/
/* next should be maxfreq*maxsetups*/
#define MAXCHANS 1000
#define AMAXSCANS 6000 /*max scans we can have in list */
#define NULL 0
#define MAXSDATA 200 /*bigger than MAXSTATIONS since may be several roots*/
/*
Changes: 
13.12.2010 - WA - Changed to right handed

 still to be done: 
>MKAFRQ  fixed at 32 but apparently not used  (nominal number of freq channels in spctrm)
>not properly provided for many freq setups..
>scale amplitude by 10000*4 approx (determined empirically..)
Changes in this version:
>sidereal rate per day +360
>-ve sign on y antenna coord
>-ve sign on clocks and rates
>only take half the given autocorr points
>correct time for first_ap number not being 0
>should work on linux with modified read_mk4fringe
>S/X possible, treated as separate setups
--next stage needed: separate corr runs for different polaris-----
--gotta fix negative decs---
--what happens if station polynoms are absent when asked for by station number ?? --
-2003: fix problems with freq list, different lengths of RR,LL and put in rotator correction -
 */
struct stat_info
    {
    char	name[32];
    char	id[3];
    int		stno;
    char	mk4_id;
    char	insert[50];
    };
struct mk4_fringe fring0,fring1,fring2,fring3; /*enuf for 4 stokes (waste of space)*/
struct type_200 *t200;
struct type_201 *t201;
struct type_202 *t202;
struct type_203 *t203;
struct type_204 *t204;
struct type_205 *t205;
struct type_206 *t206;
struct type_208 *t208;
struct type_230 *t230;
struct type_300 *t300;
struct type_301 *t301;

struct mk4_corel cdata;
struct mk4_sdata sdata[MAXSDATA];
struct stat_info idlist[MAXSTATIONS];

/* ======= extra stuff for looking at ovex etc.. ======*/
struct scan_struct *mscn;
struct scan *scn;
struct station_struct *st;
struct chan_struct *ch;
    extern int doacc, nscan;
    extern struct scan slist[];
/* -==not sure if i need these==- */
    char mymessage[81],ftnmessage[81]; /* messages to send back to aips*/
/* for reading in data files with extract*/
    fstruct *files;
    int type,  nalloc,nalloc3,  depth ;
    char data_directory [256]; /*where to look for data */
/* stuff for vxfreq */
  char setname[1550]; /* list of modes, 30char each+1 null, 50 poss*/
  int setmem[50]; /* how many members each setup*/
  int setchan[MAXCHANS]; /*setup number of each channel found*/
  int ifrq[MAXCHANS],ifrqa[MAXCHANS],ifrqp[MAXCHANS],iwhichsb[MAXCHANS],ifwant[MAXCHANS];
  double  scan_starts[AMAXSCANS], scan_stops[AMAXSCANS];
/* ====================================================*/
/* place to store collected station coords, list of frequencies etc ==*/
char antenna_name[450];        /* 8-char full name,50 members +0 between*/
char antenna_short_name[100];  /* 2*50 2-char short name */
char antenna_mk4_name[50];       /* single char name mk4*/
int antenna_found[50];          /* to check if info found already */
double antenna_x[50],antenna_y[50],antenna_z[50];  /* ant xyz coords (do I change sign of y?)*/
double tasyn[50],tasyn[50],taepo[50],taclk[50]; /* clock rubbish*/
float raxoff[50];
int iaxtype[50];
static int number_of_antennas,number_of_freqs;
static int number_of_files; /*number of stokes for this scan/baseline */
static int number_of_stokes; /*that guy stokes has a lot to answer for..*/
static float frg_snr[4],resid_sbd[4],resid_mbd[4]; /*solution snr,residuals for up to 4 stokes*/
static double resid_rate[4]; /*may need more accuracy for that..*/
static float mbd_error[4], sbd_error[4], rate_error[4], ambiguity[4], resphase[4];
static float amplitude[4], inc_seg_ampl[4], prob_false[4];
static double scan_start,scan_stop,scan_reftime; /*when this particular baseline really started, secs after ut0day1*/
static double scan_start1,scan_stop1,scan_start2,scan_stop2,scan_start3,scan_stop3; /*in case other polns different*/ 
static int kba,kba1,kba2,lkba1,lkba2;
static int sourcenum; /* number of this source as found in list sent to caller*/
static double  fourfit_ref_freq; /* need to save that to use in solutions*/
static double chanfreq[MAXCHANS]; /* collect here a table of channel freqs in Hz */
static double freqlist[MAXCHANS]; /* table of channel freqs in Hz ,not duplicated ,number_of_freq members*/
static int freq_in_setup[MAXCHANS]; /*table of setups in which these channels found*/
static int freq_num_in_setup[MAXCHANS]; /*number of these channels within setups*/
double chanbw[MAXCHANS]; /* table of channel bandwidths in Hz */
char chanpol [MAXCHANS],chansb[MAXCHANS];
char band_id [MAXCHANS]; /*  use 1st char only of band ident */
char mode_id [MAXCHANS*31]; /*mode in use, 30 char +1 *maxchans possible*/
int  first_of_sband[50]; /*if SX in one setup, offset of fsrst chan of 8/6 boundary in 16 chans*/
char sorted_band_id[MAXCHANS]; /*single-char bad id eg S,X,C Q to get fqid: list matches one give out by VXFREQ*/
char sorted_mode_id [MAXCHANS*31]; /*sorted list of modes*/
int number_of_sets;  /*sets of freq setups*/
static char source_name[21000]; /* 1000 sources * 20 character +0 between */
static int source_select[1000]; /* 0 if not selected, 1=selected, 1 entry for each in ovex*/
static int sources_in_list; /*how many sources in final list*/
static double raepoch[1000],decepoch[1000],raday0[1000],decday0[1000]; /*ra at 2000 and midnight day0 */
static int last_ap0,last_ap1,last_ap2,last_ap3;
int coordepoch[1000]; /* B1950 or J2000 */
int number_of_channels; /* collect all freq,pol,band channels */
int number_of_sources; /*collect source info */
static int first_day,first_year; /*start of first scan*/
char modedef[100]; /* what frequency mode name */
float mytai_utc, mywob_x,mywob_y,myut1_utc,my_coord_epoch;
double mygast0,mygast1,mygast_rate,my_iat0_epoch;
int mypol0,mypol1,mypol2,mypol3;
int mysideband0,mysideband1,mysideband2,mysideband3;
int nif0,nif1,nif2,nif3,ifch0[16],ifch1[16],ifch2[16],ifch3[16]; /*in crspec, ident chans*/
int global_prtlev; /*store debug level here*/
int global_coramp; /*whether to do amplitude correctioon for correlator error */
int spline_id[50]; /* for statn n in antenna_mk4_name list, look in sdata[spline_id[n]] */
 int local_start_day; /*move here for availability*/
/* =====================================================*/
int  max_seq_no;
char progname[] = "readovex_for_aips";
int msglev = 1;

#define MAXPASS 32
#define FALSE 0
#define TRUE 1
#define MAXSETUPS = 20
/* these outside so they can be accessed */
    struct vex root;
    struct vex mainovex;
    char oldroot[256], rootname[256];
/*end of moved stuff */
  char source_sel[32]; /*selection criteria */
  float time_sel_start,time_sel_stop;
/* ==================selection criteria===================================================*/
void vxtimerang_(day1,hr1,min1,sec1,day2,hr2,min2,sec2)
int *day1,*hr1,*min1,*sec1,*day2,*hr2,*min2,*sec2; {
  time_sel_start=( *day1 * 24.0 + *hr1)*3600.0 + *min1 * 60.0 + *sec1;
  time_sel_stop =( *day2 * 24.0 + *hr2)*3600.0 + *min2 * 60.0 + *sec2;
/* if all are zero probably means use all data */
  if ((time_sel_start == 0.0) && (time_sel_stop == 0.0))time_sel_stop=10.0*86400.0; /*10 day exp*/
}
/* ============================================================================= */
/* ======= call is  : call VXREFD(REFDAY,DTWOB,DTUT1,IATOFF,DTGAST,DTSIDD,IERR) 
   REFDAY is char*8, ierr is int */
void vxrefd_(refday,dtwob,dtut1,iatoff,dtgast,dtsidd,doy,ierr)
char refday[8];
double *dtwob,*dtut1,*dtgast,*dtsidd;
float* iatoff;
int *ierr,*doy;
{
  int iyear,yrfl4;
  int monday[12]={0,31,59,90,120,151,181,212,243,273,304,334};
  int daysofar,imonth,mymo,myday,iisl,iii;
  double tfl,dumm1,dumm2;
 /* start of observation is next day after eop:*/
  iyear= first_year - 1900;
  iyear= iyear - 1900;
  iisl=iyear-(iyear / 4)*4;
  for (imonth=1; imonth <13;imonth ++){
    daysofar=monday[imonth -1 ];
    if ((iisl == 0) && (imonth > 2))daysofar++;
    if ((first_day - daysofar) >0){
      myday=first_day - daysofar; mymo=imonth;
    }
  }
/* don't get GAST0 so generate it and rate for 1st day */
/* not quite sure of accuracy get 194.350982 for day 96 2001,
  ephem gives >>>>194.3468 app<<<< and 194.3512 mean.. */
/*======================================*/
  prej2(first_year,first_day,0.1,0.1,&dumm1,&dumm2,&mygast0);
  prej2(first_year,(first_day+1),0.1,0.1,&dumm1,&dumm2,&mygast1);
  mygast_rate=360.0+mygast1-mygast0; /*degrees per day */ /*+360 from Porcas sid degrees per day*/
  tfl=iyear*365.0-0.5+(iyear-1)/4+first_day;
  my_iat0_epoch=tfl+*iatoff/86400.0; /*epoch of iat 0 of first day*/
/*======================================*/
  *iatoff=mytai_utc;
  *dtwob=mywob_x; dtwob++; *dtwob=mywob_y;
  *dtut1=myut1_utc;
  sprintf (refday,"%04d%02d%02d",first_year,mymo,myday);
  *dtgast=mygast0;
  *dtsidd=mygast_rate;
  *doy=first_day;
  *ierr=0;
}
/* ============================================================================= */
/* ============ call is  :  VXANT (NANT,ANTNAM,DTALOC,DTASYN,DTAEPO,DTACLK,DTAEP1,IRERR)  */
void vxant_(nant,antnam,dtaloc,axoff,mntsta,dtasyn,dtaepo,dtaclk,dtaepo1,ierr)
int* nant;
char *antnam;
double *dtaloc,*dtasyn,*dtaepo,*dtaclk,*dtaepo1;
float *axoff;
int *mntsta;
int* ierr;
{
  int iiii;
  char ananan[8];
  *nant=number_of_antennas;     /* tell how many in ovex */
  for (iiii = 0; iiii <number_of_antennas;iiii++){
     ctoftn(antnam+iiii*8, &antenna_name[iiii*9],8); 
     *dtaloc=antenna_x[iiii]; dtaloc++;
/*      *dtaloc=-antenna_y[iiii]; dtaloc++;  /\*-ve sign of y-coord from porcas*\/ */
     *dtaloc=antenna_y[iiii]; dtaloc++;  /* sign of y-coord removed as we are now right*/
     *dtaloc=antenna_z[iiii]; dtaloc++;
     *axoff=raxoff[iiii]; axoff++;
     *mntsta=iaxtype[iiii]; mntsta++;
     *dtasyn=-tasyn[iiii]*1.0e6; dtasyn++;  /* -ve sign porcas to fits NRAO convention*/
     *dtaepo=taepo[iiii]; dtaepo++;
     *dtaclk=-taclk[iiii]; dtaclk++;  /* -ve sign porcas to fits NRAO convention*/
     *dtaepo1=taepo[iiii]; dtaepo1++; /*only 1 epoch available, have to use that*/
  }
  *ierr=0;
}
/* ============================================================================= */
/* ============ call is  :  CALL VXFREQS (NTAFQT,ITNAFQ,DTAFQT,BANDW,MKAFRQ,IFSIDE,IFPOL,IERR) */
void vxfreq_(ntafqt,itnafq,dtafqt,bandw,mkafrq,ifside,ierr)
int* ntafqt;
int *itnafq; /*no of ifs in each setup */
double *dtafqt; /* 50*16 frequencies */
float *bandw;
int *mkafrq,*ifside;
int* ierr;
{
  double *ldtafqt; /*local pointers for 2-dim arrays*/
  int *lifside;
  int iiii,i4,jjjj,kk1,kk2,isets,llll,mmmm,fos,ninset;
  char chr1;
  float testfreq;
  int *mypoint;
  float mybandw;
  int fosfac;
  number_of_freqs=0;
  fosfac=1;
 /* make list of sets of freqs, i.e. modes */
  isets=0;
  for (iiii=0;iiii < number_of_channels;iiii++){
    kk2=-1;
    for (jjjj=0;jjjj <isets;jjjj++){
      kk1=strcmp(&setname[jjjj*31] ,&mode_id[iiii*31]);
      if(kk1 == 0){
        setmem[jjjj]++; /* another member found in this set */
        kk2=jjjj;
      }
    }
    if(kk2 < 0 ){
      strcpy(&setname[isets*31] ,&mode_id[iiii*31]);
      setmem[isets]=1; /* at least 1 member of this set */
      kk2=isets;
      isets++;
    }
    setchan[iiii]=kk2; /*mark which setup this channel belongs to */
  }
  *ntafqt=isets;
  number_of_sets=isets;
  /* go through the modes, sort them in frequency and put out */
  for (llll=0;llll <isets ;llll++){
    fos=-1; /*first channel in set*/
    iiii=-1;
    for (i4=0;i4 <number_of_channels ;i4++){
      if(setchan[i4] == llll){
        if(fos < 0){ 
           fos=i4; /*mark place of first channel of this set*/
           sorted_band_id[llll]=band_id[i4]; /*1-per-mode list of band id (C,X,S,usw) */
           strcpy(&sorted_mode_id[llll*31],&mode_id[i4*31]); /*same for mode id*/
        }
        iiii++;
        ifrq[iiii]=chanfreq[i4] / 1000.0; /* integers to use in sorting */
        ifrqa[iiii]= ifrq[iiii]; 
        ifrqp[iiii]=i4-fos; /* gets sorted order of channels */
      }
    }
/* sort into ascending freq order , check poln and sideband pairs*/
    for (jjjj = 0; jjjj <iiii+1;jjjj++){
      ifwant[jjjj]=1;
      iwhichsb[jjjj]=1; /*first assume USB only */
      if(chansb[jjjj] == 'L')iwhichsb[jjjj]=-1; /* unlikely, but might be only LSB present*/
    }
    for (jjjj = 0; jjjj <iiii+1;jjjj++){  /*sort in ascending frequency order, note posn */
      for (mmmm = 0; mmmm <iiii;mmmm++){
        kk1=ifrq[mmmm]; kk2=ifrq[mmmm+1];
        if (kk1> kk2){                /*reverse pair if first > next */
          ifrq[mmmm]=kk2; ifrq[mmmm+1]=kk1;
          kk1=ifrqp[mmmm]; kk2=ifrqp[mmmm+1];
          ifrqp[mmmm]=kk2; ifrqp[mmmm+1]=kk1;
        }
      }
    }
    for (jjjj = 0; jjjj <iiii+1;jjjj++){
      for (mmmm = jjjj+1; mmmm <iiii+1;mmmm++){
        if(ifrqa[mmmm] == ifrqa[jjjj]){  /*two channels with same freq */
          ifwant[mmmm]=0; /* mark second chan as unwanted*/
          if(chansb[mmmm] != chansb[jjjj])iwhichsb[jjjj]=0; /*mark both USB and LSB specified*/
        }
      }
    }
    *mkafrq=32; /*mkafrq++; */ /* Dummy */
      /*look at first active channel to check dsb, bandwidth */
    for (mmmm = fos; mmmm <fos+iiii+1;mmmm++){
      kk1=ifrqp[mmmm];
      if(ifwant[kk1] == 1){
      mybandw=chanbw[kk1];  /*should change according to set.. */
      if (iwhichsb[kk1] == 0){
        mybandw=chanbw[kk1]*2.0; /*case of dsb, have to double bandwidth*/
        fosfac=2;
      }
      break; 
      }
    }
  /*tells later programs where eg first chan of Sband is*/ 
    if(sorted_band_id[llll] != sorted_band_id[0])first_of_sband[llll]=fos/fosfac; 
  /* set pointers */
    ldtafqt=dtafqt+llll; /*set local to external pointers +setup number */
    lifside=ifside+llll;
    *itnafq=0; /*members in setup*/
    *bandw=mybandw; bandw++;
    for (mmmm = 0; mmmm <iiii+1;mmmm++){
      kk1=ifrqp[mmmm];
      if(ifwant[kk1] == 1){
        *ldtafqt=chanfreq[kk1+fos] ; ldtafqt=ldtafqt+50;
        *lifside=iwhichsb[mmmm]; lifside=lifside+50;
        *itnafq=*itnafq+1;
        freqlist[number_of_freqs]=chanfreq[kk1+fos] ;
	freq_in_setup[number_of_freqs]=llll; /*its in this setup*/
        ninset=*itnafq; /*Sep 2004*/
	freq_num_in_setup[number_of_freqs]=ninset-1; /*number of chan  in this setup Sep 2004 */
        number_of_freqs++;
      }
    }
    itnafq++;
  } /* end of loop on sets */
  *ierr=0;
}
/* ============================================================================= */
/* call is:  SUBROUTINE SCNLST (MAXSCN, NUMSCAN,STRSCN, STPSCN, IERR)
INPUT:  1) MAXSCN: Size of STRSCN and STPSCN arrays             I

Output: 1) NUMSCAN How many scans                               I
        2) STRSCN: Time of scan start time delivered in day
           units and relative to 1st day of observation
           (MK4 def.)                                           D(MAXSCN)
        3) STPSCN: Time of scan end in day units                D(MAXSCN)
        4) IRER: Return-Status: 0 == O.K., !0 == Error          I
*/
void scnlst_(maxscn,numscan,strscn,stpscn,ierr)
int *maxscn; /* given Size of STRSCN and STPSCN arrays */
int *numscan; 
double *strscn,*stpscn; /*days, all scan start times and stop times*/
{
   int ksc;
   if (nscan > *maxscn){
     printf("need space for %d scans \n",nscan);
     ierr=-1; return ;
   }
   *numscan=nscan;
   for (ksc=0; ksc <nscan; ksc++){ 
       *strscn=scan_starts[ksc]; strscn++;
       *stpscn=scan_stops[ksc]; stpscn++;
   }
   return ;
}
/* ============================================================================= */
/* ============ call is  :  call VXSOU(NUMSRC,DESEL,SRCLST,NSOUR,SULIST,DRAEPO,DECEPO,
     >  DEPOCH,DRAAPP,DECAPP,IERR) */
void vxsou_(numsrc,desel,srclst,nsour,sulist,draepo,decepo,depoch,draapp,decapp,ierr)
int* numsrc;  /*number of sources in our input sel/desel list */
int* desel;   /*was a logical (1 is true so 0 to select, 1 deselect)*/
char srclst[480] ; /*  30 entries 16 char*/
int* nsour;
char *sulist;
double *draepo,*decepo,*depoch,*draapp,*decapp;
int* ierr;
{
 int ii,jj,kk,ll,mm,err,isour,lumsrc;
 char ftnsource[16]; /* a source name converted from c to ftn style, filled to 16 blanks*/
 char csource[17]  ; /* a source name converted from ftn to c style, terminated by zero*/
 double sfl,zeta0,zfl,theta,crz,srz,xfl,tt2,qfl,arg,rarad,decrad;
 lumsrc=*numsrc;
 isour=0;   err=1;   /*as yet no sources*/
 for(jj=0; jj < number_of_sources;jj++){  /*loop over all ovex sources*/
   source_select[jj]=*desel; /*if 0, first desel all:if 1 , select all*/
   if(*numsrc == 0)source_select[jj]=1; /*but if no srces in spec list,sel all*/
   mm= source_select[jj];  /*if found in list, change selection status*/
   for (ii = 0 ;ii < lumsrc;ii++){ /*loop over sources in sel/desel list*/
     ftntoc(&csource,&srclst[ii*16],16);
     if(strcmp(&csource[0],&source_name[jj*21]) == 0)source_select[jj]=!mm;
/* debug area , can remove */
     if(strcmp(&csource[0],&source_name[jj*21]) == 0){
     }
/*end of removable debug area */
   } 
       strcpy(sulist+isour*16,&source_name[jj*21]);
       ctoftn(sulist+isour*16,&source_name[jj*21],16);
       rarad=raepoch[jj]*3.1415926535/180.0 ; /* deg to radians*/
       decrad=decepoch[jj]*3.1415926535/180.0 ; 
       if(coordepoch[jj] == B1950 ){
         my_coord_epoch=1950.0; 
         pre50(first_year,first_day,rarad,decrad,&raday0[jj],&decday0[jj],&mygast0);
       }
       if(coordepoch[jj] == J2000 ){
         my_coord_epoch=2000.0; 
         prej2(first_year,first_day,rarad,decrad,&raday0[jj],&decday0[jj],&mygast0);
       }
       isour++;   err=0;
       *draepo=raepoch[jj]; draepo++;
       *decepo=decepoch[jj]; decepo++;
       *depoch=2000.0; *depoch++;  /*alarm*/
       *draapp=raday0[jj] *180.0/3.1415926535; draapp++;
       *decapp=decday0[jj]*180.0/3.1415926535; decapp++;
 }
 sources_in_list=isour; /*how many sources in final list*/
 *nsour=isour;
 *ierr=err;  /*will be zero if any source found*/  
}
/* ============================================================================= */
/*prtlev : 0= donothing, 1=just a scan names, 3=each baseline print ,
   write this to a routine cmsgwr (text maximum 80 characters )*/
int inivex_(fnam,expno,prtlev,coramp,iret)
char *fnam;
int *expno,*prtlev,*coramp,*iret;
{
    struct freq_corel corel[MAXFREQ];
    struct type_pass *pass;
    char *inputname, *check_rflist();
    int i,j,k,l;
    float fdiff;
    int acc_period; /* replaces param.acc_period */
    int iscan,istn,nid,ifreq;
    int longscan; /*length secs longest station in scan*/
    
    char ovex_file_name[256],cexpno[8],local_id[31];
    double decminsec;
    global_prtlev=*prtlev;               /*debugging level*/
    global_coramp=*coramp; /*whether to do amplitude correctioon for correlator error */
                                        /* Get standard environment settings */
    environment();
                                        /* Initialize IO library allocation */
       for(i =0; i < 50; i++)antenna_found[i] = 0; /* station idfound list clear */
/*data directory name construct*/
/* put together filename with path , then go read it : */
       ftntoc(ovex_file_name,fnam,48);  /*convert c string to fortran */
       sprintf(data_directory,"%s/",ovex_file_name);
       sprintf(cexpno,"/%4d",*expno); /*ovex file name*/
       strcat(ovex_file_name,cexpno);
       strcat(ovex_file_name,".ovex");
       if (get_vex (ovex_file_name, OVEX , "SCAN_LIST", &mainovex) != 0)
            {
            printf ("Error reading main ovex for file %s, skipping\n", ovex_file_name);
            }
                                        /* copy in mainovex filename for later use */
/*================================== section from stcodes ======================================*/
    if (findsiteids (idlist, &nid) != 0)
        {
        printf ("some problem in '%s'\n", ovex_file_name );
        exit (1);
        }
        number_of_antennas=nid;
        for (istn=0;istn <nid;istn++){
           antenna_mk4_name[istn]=idlist[istn].mk4_id;
           strncpy(&antenna_name[istn*9],idlist[istn].name,8);   /* 8-char full name*/
           strncpy(&antenna_short_name[istn*2],idlist[istn].id,2);  /* 2-char short name */
        }
                                                /*  make a list of the scans */
         scaninfo ("SCAN_LIST", "",modedef);
/* get all the information for scan 1 like starting epoch etc ,also mode name */
         mscn=(void *)scaninfo(slist[0].name,"",modedef);    /* points to scan 0 */
         if(mscn == 0)mscn=(void *)scaninfo(slist[1].name,"",modedef);    /* points to scan 0 */
	 if(mscn == 0){printf("incorrect return ifrom scaninfo  %x \n",mscn); return(-1);}
         first_year=mscn->start_time.year; first_day=mscn->start_time.day; 
         mytai_utc=mscn->tai_utc;
/* polar wobble translated radians->arcsec */
         mywob_x=mscn->x_wobble[1]*3600.0*180.0/3.1415926; /*first scan is on 2nd day of wob table */
         mywob_y=mscn->y_wobble[1]*3600.0*180.0/3.1415926; /* At least I damn well hope so:AUFPASSEN*/
         myut1_utc=mscn->ut1_utc[1];
                 /* now go through all the scans collecting information*/
         number_of_channels=0;  /* list all freq channels*/
         number_of_sources=0; /* list all sources*/
/*nscan comes from scan_info.c */
         for (iscan=0;iscan <nscan;iscan++){
           mscn=(void *)scaninfo(slist[iscan].name,"",modedef);    /* standard name of scan is in that list */
/*=======get modedef here======================*/
/*finish modedef section=======================*/
/*                                                   scan has mscn->nst  stations */
           longscan=0;    /*longest scan time*/ 
                 /* loop on all stations in that scan jeez what a messy way to do this===*/
           /*  ============================================================= */
           for (istn=0;istn <mscn->nst;istn++){
             st = mscn->st + istn; /* could also use mscn->st[istn].mk4_site_id */

             k=-1;    /* is this antenna in our list ? */
             for (i=0; i < nid; i++){
               if (st->mk4_site_id == antenna_mk4_name[i])k=i;  
             }
             if( k < 0){ printf(" station >%c< scn %s found in scan but not in idlist : exit \n",
                 iscan,st->mk4_site_id,slist[iscan].name);
                exit(-1);
             }
             if(st->stop_offset > longscan)longscan=st->stop_offset; /*longer?*/ 
  /* so k is the number of this station in our antenna list , if not done 
      yet then fill in information table for this antenna */
             if (antenna_found[k] == 0){ 
               antenna_found[k] = 1;
               antenna_mk4_name[k]=st->mk4_site_id;
               strncpy(&antenna_name[k*9],st->site_name,8);   /* 8-char full name*/
               strncpy(&antenna_short_name[k*2],st->site_id,2);  /* 2-char short name */
               antenna_x[k] = st->coordinates[0] ;
               antenna_y[k] = st->coordinates[1] ;
               antenna_z[k] = st->coordinates[2] ;  
               raxoff[k] = st->axis_offset;
/* VEX defines hadec=1 (AIPS type 1), azel=3 (AIPS 0), x_yns=2 (aips 4),
   x_yew =4 (aips4), use 3 for Pico (nasmyth) */
               iaxtype[k] = st->axis_type;
               if(st->axis_type == 3)iaxtype[k] =0; /*azel*/
               if(st->axis_type == 2)iaxtype[k] =4; /*azel*/
               if((fabs (st->coordinates[0] - 5088967.9) < 1000.0) &&
                 ( fabs (st->coordinates[1] +  301681.6) < 1000.0) &&
                 ( fabs (st->coordinates[2] - 3825016.0) < 1000.0))iaxtype[k] =3; /*Pico = Nasmyth !*/
/***debug****/
               tasyn[k] = st->clock_early;
               taclk[k] = st->clockrate;
               /* AUFPASSEN only seems to be one of these present, also ignore leap year*/
               /* new version refers clox to begin of current year*/
               /* but now referred to refday...*/
               taepo[k] = 
                  (st->clockrate_epoch.day - first_day)*86400.0+
                  st->clockrate_epoch.hour*3600.0+
                  st->clockrate_epoch.minute*60.0+
                  st->clockrate_epoch.second;
             }
/* Go through max 16 freq chans in this scan, add to list if unknown */
             for (ifreq =0; ifreq < MAXFREQ ;ifreq++){
               ch = st->channels + ifreq;
               if (ch->sky_frequency == F_UNDEFINED) break;
               k=-1;
               for (i=0; i < number_of_channels ;i++){
                 strncpy(local_id,ch->band_id,1); strcpy(&local_id[1],modedef); /*band+modeid*/
                 l=strcmp(local_id,&mode_id[i*31]);    
 /* 1000 in next line is to make sure Hat creek 30Hz offset frequencies not listed as new ones*/
                 if((fabs( chanfreq[i] - ch->sky_frequency) < 1000.0 )
                   &&(chanpol[i] == ch->polarization )
                   &&(chansb[i] == ch->net_sideband)
                   &&(l == 0)  )k=i;
               } /* end of looking in existing chan list loop */
               if( k < 0){
                 chanfreq[number_of_channels] = ch->sky_frequency;
                 chanbw[number_of_channels] = ch->bandwidth;
                 chanpol[number_of_channels] = ch->polarization;
                 chansb[number_of_channels] = ch->net_sideband;
                 strncpy(&band_id[number_of_channels], ch->band_id,1); /*  1-char idents */
   /* now use mode IDs which include band eg Xmymode, Smymode*/
                 strncpy(&mode_id[number_of_channels*31],ch->band_id,1); 
                 strcpy(&mode_id[number_of_channels*31+1] , modedef); /* mode ident */
                 number_of_channels++;
                 if (number_of_channels > MAXCHANS ){
                   printf("too many channels \n");
                   exit(-1);
                 }
               }
             } /* end of looking for freq chans loop */
           }   /* end of looking in station list loop*/
       /*work out start and latest stop for that scan ,fill array*/
           scan_starts[iscan]=(mscn->start_time.day - first_day) + mscn->start_time.hour/24.0+
                               mscn->start_time.minute/1440.0+mscn->start_time.second/86400.0;
           scan_stops[iscan]=scan_starts[iscan]+longscan/86400.0; 
       /* go looking for sources ====================*/
           k=-1;
           for (i=0; i < number_of_sources ;i++){
             l=strcmp(mscn->src.source_name,&source_name[21*i]);
             if (l == 0)k=i;
           }
           if (k < 0){
             strcpy(&source_name[21*number_of_sources],mscn->src.source_name);
             coordepoch[number_of_sources] = mscn->src.position_ref_frame; /* B1950 or J2000 */
             raepoch[number_of_sources]=mscn->src.position.ra_hrs*15.0   /*in degrees*/
                                 +mscn->src.position.ra_mins/4.0
                                 +mscn->src.position.ra_secs/240.0     ; 
       /*following takes care of negative declination ,parsedec fixes btwn 00 and -01! */
             decepoch[number_of_sources]=mscn->src.position.dec_degs;     /*in degrees*/
             decminsec=mscn->src.position.dec_mins/60.0
                      +mscn->src.position.dec_secs/3600.0 ;
             if(decepoch[number_of_sources] < 0.0)decminsec = -decminsec;
             decepoch[number_of_sources]=decepoch[number_of_sources] + decminsec;
             number_of_sources++;
           }
         } /* end of scanlist loop */
    *iret=0;
    return(0);
    }
/* ============================================================================= */
/* ==== call is : CRSPEC(MXTVIS,TUTC,IA1,IA2,IAREF,IFQAID,CURSOU,
     >                      CURINT,ISTOKE,DSB,CROSSP,CRSLGT,IRET)
 ================================================================================*/
int crspec_(mxtvis,tutc,ia1,ia2,iaref,ifqaid,cursou,curint,istoke,
            fqid,crossp,crslgt,iret)
int *mxtvis,*ia1,*ia2,*iaref,*ifqaid,*cursou,*istoke,*fqid,*crslgt,*iret;
double *tutc;
float *crossp, *curint;
{
/* all the following static things so stay set between calls*/
 static int stokes_left=-1; /* how many stokes to come for this ap and if*/
 static int ifs_left=-1; /* how  active ifs to come for this ap*/
 static int points_left=-1; /* how many aps left to come ? :-ve means new scan needed*/
 static int points_left0,points_left1,points_left2,points_left3,which_longest; 
 static int scans_left=-1; /*how many scans left ? :-ve means initialize list*/
 static int baselines_left=-1; /* <=0 if all baselines in this scan are done, need new scan*/
 static int ibase[1000]; /*Aug 07 increase from 200*/
 static int iscan; /* scan number we are doing*/
 static int filenum=0,filenums=0; /*number of file in scan we are doing*/
 static int recno;  /* number of ap currently */
 static int number_of_ifs;   /*number of ifs active*/
 static double time_per_ap,time_per_ap1,time_per_ap2,time_per_ap3;
 static int firstap0,firstap1,firstap2,firstap3;
 static int nspec_pts; /*how many points in spectrum*/
 static int ka1[1000],ka2[1000],kbase,kfcode; /*antenna ids for each file in a directory */
 static int which_stokes,which_if;
 static int local_fqid; /*local store for fqid*/
 static char frg_qual[4]; /*to judge solutions by*/
 static double local_utc; 
 static int firsttime=1; /*first go so intialize some things*/
 double ttdd; /* use fotdime diff now-fourfit ref time*/
 int i,ii,k,l,m,n,kfqaid,npts,nn;
 char stn1,stn2; /*mk4 letters*/
 char dirname[256],file_id[8],first_id[8]; 
 char ant1[2],ant2[2]; /*store antenna names: goes quicker this way*/
 float scan_nominal_start; /*inaccurate start time in secs after day1 midnight*/
 float aptimeshort;
 int iif,idummy;
 int len230off;
 int nxspec_pts; /* = nspecpts or half of it */
 char modeuse[31]; /* what frequency mode name used in this scan */
 char local_id[31];
 static double fudgefactor; /* to fix correlator errors before march 2003*/
 double geodel1[4],geodel2[4]; /* interpolated model, needed for correlator error correction */
 double phasdev,pacer,tote,dtee,phsr,vvfac,phasaccel; /*for correlator error fix */
 double dflo,kfactor,deltaphdot,fsample; /*more stuff for correlator error*/
 static double corrfac2,corrfac3;
 static int local_crspec_iret=0; /*to check for accidental reentry after data is finished */
 int kphas;
 if( firsttime == 1){ 
    fring0.nalloc=0; fring1.nalloc=0;fring2.nalloc=0;fring3.nalloc=0;firsttime=0;
    for (l=0; l<MAXSDATA; l++){ sdata[l].nalloc=0; clear_mk4sdata (&sdata[l]); } 
 }
 /* make sure no accidental reentry disaster*/
 if (local_crspec_iret == -1){
    sprintf(mymessage," attempted reentry after data end");
    ctoftn(ftnmessage,mymessage,80);
    cmsgwr_(ftnmessage);  /*send a message back to aips */
    *iret=1;
    return(0);
 }
 back_to_start:    /*go here if want to avoid returning invalid data*/
 *iret=0; /* assume for the moment nothing unusual happens*/
 if(stokes_left <=0){ /*means: if + still stokes left to do*/
   if(ifs_left <=0 ){ /*means: if + there are still ifs to do*/
      if (points_left <= 0){  /* means: need to get a baseline*/
        baselines_left=0; /*check if all baselines of this scan done*/
        for (l=0; l< filenum; l++) {
          if((ibase[l] >= 0) && (ibase[l] < 900000))baselines_left++;  /*not an unwanted base ?*/
        }
        if (baselines_left <= 0){ /*all baselines finished, need next scan*/
          if (scans_left < 0 )iscan=-1;  /*new scan list, call for first scan*/
label1:
          iscan++; /*point to next scan*/
          scans_left=nscan-iscan;
          if(scans_left == 0){*iret=-1;  local_crspec_iret=*iret; return(0);} /* prhaps finished*/
          mscn=(void *)scaninfo(slist[iscan].name,"",modedef);    
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/* is this source selected ? */
          k=-1;
          for (i=0; i < number_of_sources ;i++){
            if(source_select[i] !=0 ){ /*selected?*/
              l=strcmp(mscn->src.source_name,&source_name[21*i]);
              if (l == 0)k=i;
            }
          }
          if(k < 0)goto label1; /*not selected*/
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
          scan_nominal_start=(mscn->start_time.day-first_day)*86400.0
                      +mscn->start_time.hour*3600.0
                      +mscn->start_time.minute*60.0
                      +mscn->start_time.second;
/*
 is this scan time in selected range ?*/
          if (scan_nominal_start < time_sel_start){goto label1;} /*Aug07*/
          if (scan_nominal_start > time_sel_stop){*iret=-1; local_crspec_iret=*iret; return(0);} /* HEH prhaps finished*/ 
          if ((scan_nominal_start >= time_sel_start) && (scan_nominal_start <time_sel_stop)){
               strcpy(dirname,data_directory);
               strcat(dirname,slist[iscan].name); /* put together dir name where data is*/
/*----------------type3 (splines) section------------------*/
               if( filenums != 0){             /* if space for sdata allocated last scan, clear it */
                 for (l=0; l<filenums; l++){
                     clear_mk4sdata (&sdata[l]);
                 }
               }
               filenums=0;
               k=extract_filenames (dirname, 3, &files, &nalloc3, &filenums, &depth) ; /*type 3 files only*/
	       if (k  ==0 ){
                 for (l=0; l< filenums; l++) {
                   if (read_mk4sdata (files[l].name, &sdata[l]) != 0)printf("sdata: problem file: ",files[l].name);
                   t300=sdata[l].t300; 
                   for (m=0; m < number_of_antennas; m++){  /*identify what station in list assoc with this spline file*/
                       if (t300->id == antenna_mk4_name[m])spline_id[m]=l;                     
                   }
                   if( k < 0)printf(" station >%c< scn %s in splines but not in idlist \n", t300->id,files[l].name);
                 }
	       }
/*-----------end--type3 spline section------------------*/
               filenum=0; /*Aug07*/
               k=extract_filenames (dirname, 2, &files, &nalloc, &filenum, &depth) ; /*type 2 files only*/
	       if (k == 0 ){
/* debug: might want following statement as a msg to AIPS*/
                 if(global_prtlev >= 1 ){
                     sprintf(mymessage," %s file name extract, #files=:%d: %d:",dirname,k,filenum);
                     ctoftn(ftnmessage,mymessage,80);
                     cmsgwr_(ftnmessage);  /*send a message back to aips */
                 }
	       }
               if(k !=0 ){goto label1;} /*no data directory or files for this scan*/
               if(filenum < 1 ){goto label1;} /*no  files for this scan*/
               for (l=0; l< 1000; l++) {
                 ka1[l]=-1; ka2[l]=-1; ibase[l]=-1;
               }
               for (l=0; l< filenum; l++) {
                 ibase[l]=999999;
                 stn1=files[l].baseline[0]; stn2=files[l].baseline[1];
                 for (k=0; k< number_of_antennas;k++){   /*identify antennas from list*/
                     if(stn1 == antenna_mk4_name[k] )ka1[l]=k;
                     if(stn2 == antenna_mk4_name[k] )ka2[l]=k;
                     ibase[l]=ka1[l]*1000+ka2[l]; /*AUFPASSEN ignore auto for moment*/
                 }
/* debug: might want following statement as a msg to AIPS*/
                 if(global_prtlev >= 2 ){
                   sprintf(mymessage," %03d %06d %s %c %d %s",l,ibase[l],files[l].baseline,
                           files[l].freq_code, files[l].type,files[l].name);
                   ctoftn(ftnmessage,mymessage,80);
                   cmsgwr_(ftnmessage);  /*send a message back to aips */
                 }
               }
          }  /* end of scan start checking loop*/
        }     /*end of (new) loop that looks for undone baselines*/
           number_of_files=0;   /* from 1-4, e.g. stokes , number of type 2s for this base*/
          fring0.n230=0; fring1.n230=0;fring2.n230=0;fring3.n230=0;
           for (l=0; l< filenum; l++) {  /*go through looking for start of next base*/
             if (ibase[l] >= 0 ){  
                kbase=ibase[l];
                kba=kbase/1000; kba1=kba+1; kba2=kbase-1000*kba+1; /*antenna ids, ftn numbering*/
                kfcode=files[l].freq_code; /*make sure separate freq codes (SX) not read in simultan*/
                goto weiter;
             }
           }
           weiter:
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++free space++++++++++++++++++*/
           if(fring0.nalloc !=0){for(i=0; i<fring0.nalloc; i++){free (fring0.allocated[i]);}fring0.nalloc=0;}
           if(fring1.nalloc !=0){for(i=0; i<fring1.nalloc; i++){free (fring1.allocated[i]);}fring1.nalloc=0;}
           if(fring2.nalloc !=0){for(i=0; i<fring2.nalloc; i++){free (fring2.allocated[i]);}fring2.nalloc=0;}
           if(fring3.nalloc !=0){for(i=0; i<fring3.nalloc; i++){free (fring3.allocated[i]);}fring3.nalloc=0;}
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++end free space++++++++++++++++++*/
           for (m=0; m< filenum; m++) {  /*open up to 4 files belonging to that base */  
             if ((ibase[m] == kbase) && (kfcode == files[m].freq_code)){   
               if (kbase != 999999){          /*i.e. if this is a valid baseline*/
                 if(number_of_files == 0){
                    n= read_mk4fringe (files[m].name, &fring0);
                    if(n !=0)printf ("Error return from read_mk4fringe %d \n",n); 
                    local_fqid=-1;
                    for (nn=0; nn< number_of_sets; nn++){
                    sprintf(local_id,"%c%s",files[l].freq_code,modedef); /*find band+modeid*/
                    if(strcmp(local_id,&sorted_mode_id[nn*31]) ==0)local_fqid=nn;    
                    }
                 }
                 if(local_fqid < 0){
                     local_fqid=0;      /*if not found, asume its setup 1*/
                     printf("readvex: band mismatch: check ovex chan_def \n");
                 }
                 if(number_of_files == 1){
                    n= read_mk4fringe (files[m].name, &fring1);
                    if(n !=0)printf ("Error return from read_mk4fringe %d \n",n);  
                 }
                 if(number_of_files == 2){
                    n= read_mk4fringe (files[m].name, &fring2);
                    if(n !=0)printf ("Error return from read_mk4fringe %d \n",n);  
                 }
                 if(number_of_files == 3){
                    n= read_mk4fringe (files[m].name, &fring3);
                    if(n !=0)printf ("Error return from read_mk4fringe %d \n",n); /*goto erret;*/ 
                 }
                 number_of_files++;
               }
               ibase[m]=-1;   /*mark opened i.e. done*/  
             }
           }
           if(number_of_files >4){
              printf("debug #files %d is >4 , use aedit to fix \n",number_of_files);
              number_of_files = 4;
           }
           if(number_of_files >0){
               t201=fring0.t201;
               sourcenum=-1;
               for(m=0; m < number_of_sources; m++){
                if(strcmp(t201->source,&source_name[m*21]) == 0)sourcenum=m;  /*found in list ? */
               }
               t205=fring0.t205; t206=fring0.t206;
/*               local_start_day=t205->start.day; */ /*can be kaputt on linux, so take stop instead*/
               local_start_day=t205->stop.day;
     /*does scan go over midnight ?*/
               if( (t205->stop.hour - t205->start.hour) < 0.0)local_start_day--;
               fourfit_ref_freq=t205->ref_freq;
               t200=fring0.t200; 
     /*fourfit reftime in usual unit:secs from ut midnight of day 1 of experiment*/
               scan_reftime=(t200->frt.day-first_day)*86400.0+t200->frt.hour*3600.0
                          +t200->frt.minute*60.0+t200->frt.second;
               scan_start=(local_start_day-first_day)*86400.0 +t205->start.hour*3600.0
                          +t205->start.minute*60.0 +t205->start.second;
               scan_stop=(t205->stop.day-first_day)*86400.0 +t205->stop.hour*3600.0
                          +t205->stop.minute*60.0 +t205->stop.second;
               time_per_ap=(scan_stop-scan_start)/(t206->last_ap-t206->first_ap);
	       firstap0=t206->first_ap;
               t203=fring0.t203; t205=fring0.t205; identify_chan(t203,t205,&mypol0,&mysideband0,&nif0,&ifch0); 
               t208=fring0.t208; frg_qual[0]=t208->quality; frg_snr[0]=t208->snr; 
               resid_rate[0]=t208->resid_rate; resid_sbd[0]=t208->resid_sbd;resid_mbd[0]=t208->resid_mbd; 
               mbd_error[0]=t208->mbd_error; sbd_error[0]=t208->sbd_error; rate_error[0]=t208->rate_error;
               ambiguity[0]=t208->ambiguity; resphase[0]=t208->resphase; amplitude[0]=t208->amplitude;
               inc_seg_ampl[0]=t208->inc_seg_ampl; prob_false[0]=t208->prob_false;
               if (number_of_files >1){
                 t203=fring1.t203; t205=fring1.t205; identify_chan(t203,t205,&mypol1,&mysideband1,&nif1,&ifch1); 
		 t206=fring1.t206;
                 scan_start1=(local_start_day-first_day)*86400.0 +t205->start.hour*3600.0
                          +t205->start.minute*60.0 +t205->start.second;
                 scan_stop1=(t205->stop.day-first_day)*86400.0 +t205->stop.hour*3600.0
                          +t205->stop.minute*60.0 +t205->stop.second;
                 time_per_ap1=(scan_stop1-scan_start1)/(t206->last_ap-t206->first_ap);
	         firstap1=t206->first_ap;
                 t208=fring1.t208; frg_qual[1]=t208->quality; frg_snr[1]=t208->snr; 
                 resid_rate[1]=t208->resid_rate; resid_sbd[1]=t208->resid_sbd;resid_mbd[1]=t208->resid_mbd; 
                 mbd_error[1]=t208->mbd_error; sbd_error[1]=t208->sbd_error; rate_error[1]=t208->rate_error;
                 ambiguity[1]=t208->ambiguity; resphase[1]=t208->resphase; amplitude[1]=t208->amplitude;
                 inc_seg_ampl[1]=t208->inc_seg_ampl; prob_false[1]=t208->prob_false;
                 if (time_per_ap1 != time_per_ap)printf("warning! time per ap not same as previous n230=%f:%f \n",time_per_ap1, time_per_ap);
               }
               if (number_of_files >2){
                 t203=fring2.t203; t205=fring2.t205; identify_chan(t203,t205,&mypol2,&mysideband2,&nif2,&ifch2); 
		 t206=fring2.t206;
                 scan_start2=(local_start_day-first_day)*86400.0 +t205->start.hour*3600.0
                          +t205->start.minute*60.0 +t205->start.second;
                 scan_stop2=(t205->stop.day-first_day)*86400.0 +t205->stop.hour*3600.0
                          +t205->stop.minute*60.0 +t205->stop.second;
                 time_per_ap2=(scan_stop2-scan_start2)/(t206->last_ap-t206->first_ap);
	         firstap2=t206->first_ap;
                 t208=fring2.t208; frg_qual[2]=t208->quality; frg_snr[2]=t208->snr; 
                 resid_rate[2]=t208->resid_rate; resid_sbd[2]=t208->resid_sbd;resid_mbd[2]=t208->resid_mbd; 
                 mbd_error[2]=t208->mbd_error; sbd_error[2]=t208->sbd_error; rate_error[2]=t208->rate_error;
                 ambiguity[2]=t208->ambiguity; resphase[2]=t208->resphase; amplitude[2]=t208->amplitude;
                 inc_seg_ampl[2]=t208->inc_seg_ampl; prob_false[2]=t208->prob_false;
                 if (time_per_ap2 != time_per_ap)printf("warning! time per ap not same as previous n230=%f:%f \n",time_per_ap2, time_per_ap);
               }
               if (number_of_files >3){
                 t203=fring3.t203; t205=fring3.t205; identify_chan(t203,t205,&mypol3,&mysideband3,&nif3,&ifch3); 
		 t206=fring3.t206;
                 scan_start3=(local_start_day-first_day)*86400.0 +t205->start.hour*3600.0
                          +t205->start.minute*60.0 +t205->start.second;
                 scan_stop3=(t205->stop.day-first_day)*86400.0 +t205->stop.hour*3600.0
                          +t205->stop.minute*60.0 +t205->stop.second;
                 time_per_ap3=(scan_stop3-scan_start3)/(t206->last_ap-t206->first_ap);
	         firstap3=t206->first_ap;
                 t208=fring3.t208; frg_qual[3]=t208->quality; frg_snr[3]=t208->snr; 
                 resid_rate[3]=t208->resid_rate; resid_sbd[3]=t208->resid_sbd;resid_mbd[3]=t208->resid_mbd; 
                 mbd_error[3]=t208->mbd_error; sbd_error[3]=t208->sbd_error; rate_error[3]=t208->rate_error;
                 ambiguity[3]=t208->ambiguity; resphase[3]=t208->resphase; amplitude[3]=t208->amplitude;
                 inc_seg_ampl[3]=t208->inc_seg_ampl; prob_false[3]=t208->prob_false;
                 if (time_per_ap3 != time_per_ap)printf("warning! time per ap not same as previous n230=%f:%f \n",time_per_ap3, time_per_ap);
               }
/* OOOOOOOOOOOOOproper treatment of data from different correlations which have differnt lengthsOOOOOOOOOOOOOOOOOOOOOOO*/
               t205=fring0.t205;t206=fring0.t206;  last_ap0=t206->last_ap;
               number_of_ifs=fring0.n212;
               points_left=fring0.n230/fring0.n212; /* AUFPASSEN total number of aps we have to put out*/
	       which_longest=0;

	       points_left0=points_left; /* keep a copy... */
               if (number_of_files >1){
                  t205=fring1.t205;t206=fring1.t206;  last_ap1=t206->last_ap;
                  if (fring1.n212 > number_of_ifs)number_of_ifs=fring1.n212;
                  points_left1=(fring1.n230/fring1.n212); 
                  if ((points_left1) > points_left){points_left=points_left1; which_longest=1;}
               }
               if (number_of_files >2){
                 t205=fring2.t205;t206=fring2.t206;  last_ap2=t206->last_ap;
                  if (fring2.n212 > number_of_ifs)number_of_ifs=fring2.n212;
                  points_left2=(fring1.n230/fring1.n212); 
                  if ((points_left2) > points_left){points_left=points_left2; which_longest=2;}
               }
               if (number_of_files >3){
                 t205=fring3.t205;t206=fring3.t206;  last_ap3=t206->last_ap;
                  if (fring3.n212 > number_of_ifs)number_of_ifs=fring3.n212;
                  points_left3=(fring1.n230/fring1.n212); 
                  if ((points_left3) > points_left){points_left=points_left3; which_longest=3;}
               }

               t205=fring0.t205;t206=fring0.t206;
/*ALARMALARM following nbg for spectral crap*/
               nif0 =number_of_ifs;
               ifs_left=nif0;   
/* OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO*/
               number_of_stokes=number_of_files; /*these are really the same thing..*/
               ifs_left=number_of_ifs;
               stokes_left=number_of_files;
               number_of_files=0; /*set pointer to first fringe back to null*/
               recno=-1;
               *iret=-10; /*signal new scan, new solution to be fetched */
               local_crspec_iret=*iret; 
           } /* end of loop that opens new data files*/
      } /*end of loop that checks for aps left*/
      recno++;
      if(which_longest == 0)t230=fring0.t230[recno]; 
      if(which_longest == 1)t230=fring1.t230[recno]; 
      if(which_longest == 2)t230=fring2.t230[recno]; 
      if(which_longest == 3)t230=fring3.t230[recno]; 
      nspec_pts=t230->nspec_pts; 
      local_utc=(scan_start+time_per_ap*(t230->ap - t206->first_ap))/86400.0;  /* Porcas ap correction */
      *tutc=local_utc;
      points_left--; /* mark that this ap is going out*/ 
      ifs_left=number_of_ifs;
    } /* end of loop that checks ifs left*/
    ifs_left--;
    stokes_left=number_of_stokes;
  } /* end of loop that checks stokes left*/
  which_if=number_of_ifs-ifs_left-1;
  which_stokes=number_of_stokes-stokes_left;
  stokes_left--;
/* ==========================copy data section======================= */
/* OOOOOO/OOOOOOOOOOOOO/goto back_to_start   to avoid returning bad data...*/
/* (((((((((((((((((((((((())))))))))))))))))(((((((((((((((((()))))))*/
  *ia1=kba1; *iaref=kba1; *ia2=kba2; /*antenna ids, ftn numbering*/
  if(kbase > 900000)goto back_to_start; /* this is bum, invalid data, don't return*/
  nxspec_pts=nspec_pts;
  if( *ia1 == *ia2 )nxspec_pts=nspec_pts/2;  /*autocorrelation case : twice as many points, cut them down */
  if ((nxspec_pts*number_of_files) > *mxtvis ){
      printf("too many points for array \n");
      *iret=1; local_crspec_iret=*iret; return(0);
  }
     /* set pointer to correct fringe array and stokes id */
  if(which_stokes == 0){
     len230off= fring0.n230/number_of_ifs; 
     if ((recno >= points_left0 ) || (recno >= last_ap0))goto back_to_start;
     t230=fring0.t230[recno+which_if*len230off]; *istoke=mypol0; 
     local_utc=(scan_start+time_per_ap*(t230->ap - firstap0))/86400.0;  /* Porcas ap correction */
     *ifqaid=ifch0[t230->frq]+1; /* id number of which of 16 if channels (ftn convention)*/
  }
  if(which_stokes == 1){
     len230off= fring1.n230/number_of_ifs; 
     if ((recno >= points_left1 ) || (recno >= last_ap1))goto back_to_start;
     t230=fring1.t230[recno+which_if*len230off]; *istoke=mypol1;
     local_utc=(scan_start1+time_per_ap1*(t230->ap - firstap1))/86400.0;  /* Porcas ap correction */
     *ifqaid=ifch1[t230->frq]+1; 
  }
  if(which_stokes == 2){
     len230off= fring2.n230/number_of_ifs; 
     if ((recno >= points_left2 ) || (recno >= last_ap2))goto back_to_start;
     t230=fring2.t230[recno+which_if*len230off]; *istoke=mypol2;
     local_utc=(scan_start2+time_per_ap2*(t230->ap - firstap2))/86400.0;  /* Porcas ap correction */
     *ifqaid=ifch2[t230->frq]+1; 
  }
  if(which_stokes == 3){
     len230off= fring3.n230/number_of_ifs; 
     if ((recno >= points_left3 ) || (recno >= last_ap3))goto back_to_start;
     t230=fring3.t230[recno+which_if*len230off]; *istoke=mypol3;
     local_utc=(scan_start3+time_per_ap3*(t230->ap - firstap3))/86400.0;  /* Porcas ap correction */
     *ifqaid=ifch3[t230->frq]+1; 
  }
  *tutc=local_utc; /* moved this here July 05*/
  if (global_coramp == 0){
    for (i=0; i<nxspec_pts; i++)
      {
         ii=i; if( *ia1 == *ia2 )ii=i*2+1; /*autocorr case ,skip half the points*/
         *crossp= t230->xpower[ii].re; crossp++;
         *crossp= t230->xpower[ii].im; crossp++;
         if (i < (nxspec_pts/2)) *crossp= t230->lsbweight; 
                          else  *crossp= t230->usbweight; 
         crossp++;
      }
  }
  /* unroll this a little so we dont make it slower in standard case */
  /*CCCCCCCCCCCCCCCCCCCCCCCCCCCCCcorrpol section see RJC memo of 2003.2.12 CCCCCCCCCCCCCCCCCCCCCCCC*/
  if (global_coramp >= 1){
    if((kba1 != lkba1 ) || (kba2 != lkba2 )){  /*check if new scan,indicated by new baseline */
        lkba1=kba1; lkba2=kba2;
	cmpmdl_(&kba1,&local_utc,geodel1,&idummy);
	cmpmdl_(&kba2,&local_utc,geodel2,&idummy);
	phasaccel=t203->channels[0].ref_freq * (geodel1[2]-geodel2[2]); /*freq + accel */
	pacer=0.5*phasaccel*6.28;
/* The "Problem 2" calculation done on foot
 probable phase accel error half total rad/s/s */
/*phasdev is accumulated phase err in this frame (rads) */
        tote=0; phasdev=0;
/*split corr frame into 10 steps 50 millisec each */
        for (kphas=0 ; kphas < 10 ; kphas++){
            dtee=0.05*kphas; 
	    phsr=pacer*(dtee-0.025);
	    phasdev=phasdev+phsr*0.05;
	    vvfac=cos(phasdev);
	    if(vvfac < 0){vvfac=0;} /* if loss < 0 then just ignore */
	    tote=tote+vvfac;
        }
	corrfac2=tote/10;
/* Problem 3 correction according to rates*/
        dflo=t203->channels[0].ref_freq-t203->channels[0].rem_freq;
/* if sample 32MHz k=1 etc. (is that correct ?) */
 /* samplerate in strange units, 18432 =32Ms/s,2304 = 4 etc */
	fsample=t203->channels[0].sample_rate*1.0e6 /576.0 ; 
	kfactor=32.0e6/fsample; 
        deltaphdot=kfactor/32.0e6 * (dflo+(geodel1[1]-geodel2[1])*t203->channels[0].ref_freq)*geodel1[1]*fsample;
        deltaphdot=6.28*deltaphdot;
        corrfac3=1.0; if(kba1 != kba2)corrfac3=sin(0.5*deltaphdot)/(0.5*deltaphdot);
        fudgefactor=corrfac2; if(global_coramp > 1)fudgefactor=(corrfac2*corrfac3);
        fudgefactor=1.0/fudgefactor;
    }
    for (i=0; i<nxspec_pts; i++)
      {
         ii=i; if( *ia1 == *ia2 ){
		 ii=i*2+1; /*autocorr case ,skip half the points*/
		 fudgefactor=1.0;
	 }
         *crossp= t230->xpower[ii].re * fudgefactor ; crossp++;
         *crossp= t230->xpower[ii].im * fudgefactor ; crossp++;
         if (i < (nxspec_pts/2)) *crossp= t230->lsbweight; 
                          else  *crossp= t230->usbweight; 
         crossp++;
      }
  }
  /*CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC*/
  *cursou=sourcenum+1; /* 0->#1 since ftn is calling*/
  aptimeshort =time_per_ap;
  *curint=aptimeshort; /*just in case, float not double..*/
  *crslgt=nxspec_pts;
  *fqid=local_fqid+1; /*fortran !*/
  local_crspec_iret=*iret; /*keep local copy */
  return(0);
/* (((((((((((((((((((((((())))))))))))))))))(((((((((((((((((()))))))*/
/* ================================================================== */
}
/* +=================================================================+ */
/* this looks at 203/205, finds how many IF chans and their frequency, which
   number in our frequency list, checks if dsb or usb, which Stokes */
int identify_chan(x203,x205,pol,sideband,nifs,ifids)
int *pol,*sideband,*nifs,*ifids;
struct type_203 *x203;
struct type_205 *x205;
{
/* pol (ref,remote)=1 RR 2 LL 3 RL 4 LR, sideband=1 for DSB, 0 for USB */
   int i,j,k,m;
   *sideband=1; /*first assume only USB*/
   *nifs=0;
   for (i=0; i<16;i++){
     if (x205->ffit_chan[i].ffit_chan_id != ' '){ /* In use? */ 
     for (j = 0; j < 4; j++) { /* Each channel index */ 
        if (x205->ffit_chan[i].channels[j] >= 0){ /* In use? */ 
          m=x205->ffit_chan[i].channels[j];
        }
     }
     for (k=0; k<32; k++){
        if(k == m){
           *pol=1; /* first try:RR */
           if (x203->channels[k].refpol != x203->channels[k].rempol)*pol=3; /*Xpol */
           if (x203->channels[k].refpol == 'L')*pol=*pol+1;
           if (x203->channels[k].refsb =='L')*sideband=0; /*if any LSB, assume all DSB*/
         for(j=0; j< number_of_freqs;j++){
             if(( x203->channels[k].ref_freq == freqlist[j]) || 
                ( x203->channels[k].rem_freq == freqlist[j]) ) {
                *ifids=freq_num_in_setup[j]; /*what numbered chan in this setup*/
            /*    *ifids=j; */  /*output array with list of freq channels identified*/
                *ifids++;
                *nifs=*nifs+1;
             }
           }
        }
     }
     }
   }
}
/* =============================================== */
/*cpy ftn string p2 terminated by space to c string p1 term by null */
int ftntoc (p1, p2,nsp)
      char *p1;
      char *p2;
      int nsp;
{
  unsigned char *s1 = ( unsigned char *) p1;
  unsigned char *s2 = ( unsigned char *) p2;
  unsigned char  c2;
  int nn;
  nn=nsp;
  while (nn > 0)
    {
      c2 = (unsigned char) *s2++;
      *s1++=c2;
      if (c2 == '\x20'){ 
        *--s1='\0'; 
         return (0);
      }
      nn--;
    }
  return (0);
}
/*cpy c string p2 terminated by null to ftn string p1 term term
    by space and padded to nsp spaces */
int ctoftn (p1, p2,nsp)
      char *p1;
      char *p2;
      int nsp;
{
  unsigned char *s1 = ( unsigned char *) p1;
  unsigned char *s2 = ( unsigned char *) p2;
  unsigned char  c2;
  int nn;
  nn=nsp;
  while (nn > 0)
    {
      c2 = (unsigned char) *s2++;
      *s1++=c2;
      if (c2 == '\x0'){ 
        *--s1='\x20'; 
         while (nn > 0){*s1++='\x20'; nn--; } /*pad */
         return (0);
      }
      nn--;
    }
  return (0);
}
/*+=========================================================================*/
/*istation is (ftn)number in list of all antennas, 
     tsec is time to calculate, in secs from ut midnight of first expt day*/
int cmpmdl_(istation,tsec,geodly,istatus)
int *istation,*istatus;
double *tsec,*geodly;
{
  int i,ii,mm;
  double x_mod_start,  x_tdiff;
  double x_int_no, x_t, x_t2, x_t3, x_t4, x_t5;
  double x_doppler;
  double floor();
 struct mk4_sdata *xsd;
  struct type_301 *x301;
  ii=*istation; i=spline_id[ii-1]; t300=sdata[i].t300; 
    x_mod_start = 86400.0 * (double)t300->model_start.day
                        + 3600.0 * (double)t300->model_start.hour
                        + 60.0 * (double)t300->model_start.minute
                        + (double)t300->model_start.second;
     x_tdiff=*tsec; /* this is time in secs from ut0 of day of start of experiment*/
     x_tdiff=(x_tdiff+first_day)*86400.0 -x_mod_start ; /*or from ref_time ?*/
     x_int_no = floor (x_tdiff / (double)t300->model_interval);
    for (mm=0; mm<t300->nsplines; mm++)if (sdata[i].model[0].t301[mm]->interval == (int)x_int_no) break;
    if (mm == t300->nsplines)
      {
        *istatus=-1;  /*no spline available*/
        return (-1);
      }
    x_t = x_tdiff - (x_int_no * t300->model_interval);
                                        /* Powers of t */
    x_t2 = x_t * x_t;
    x_t3 = x_t2 * x_t;
    x_t4 = x_t3 * x_t;
    x_t5 = x_t4 * x_t;
                                        /* Compute delays */
    *geodly =         sdata[i].model[0].t301[mm]->delay_spline[0]
                    + sdata[i].model[0].t301[mm]->delay_spline[1] * x_t
                    + sdata[i].model[0].t301[mm]->delay_spline[2] * x_t2
                    + sdata[i].model[0].t301[mm]->delay_spline[3] * x_t3
                    + sdata[i].model[0].t301[mm]->delay_spline[4] * x_t4
                    + sdata[i].model[0].t301[mm]->delay_spline[5] * x_t5;
     geodly++;
                                       /*compute delay rates*/
    *geodly =         sdata[i].model[0].t301[mm]->delay_spline[1]
                    + sdata[i].model[0].t301[mm]->delay_spline[2] * 2.0 * x_t
                    + sdata[i].model[0].t301[mm]->delay_spline[3] * 3.0 * x_t2
                    + sdata[i].model[0].t301[mm]->delay_spline[4] * 4.0 * x_t3
                    + sdata[i].model[0].t301[mm]->delay_spline[5] * 5.0 * x_t4;
     geodly++;
                                        /* Compute accelerations */
    *geodly = 2.0 *   sdata[i].model[0].t301[mm]->delay_spline[2]
                    + sdata[i].model[0].t301[mm]->delay_spline[3] * 6.0 * x_t
                    + sdata[i].model[0].t301[mm]->delay_spline[4] * 12.0 * x_t2
                    + sdata[i].model[0].t301[mm]->delay_spline[5] * 20.0 * x_t3;
                                        /* Baseline apriori model */
   *istatus=0;
   return(0);
}
/*======================================================================================*/
int getsol_(stokes,baseln,cursou,tuts,intts,vamp,samp,rmbd,mbderr,mbdamb,rrate,rterr,
            rsbd,sbderr,rphase,phserr,probf,irets)
double *tuts; /*ut time of solution*/
float *intts,*vamp,*samp,*rmbd,*mbderr,*mbdamb,*rrate,*rterr,*rsbd,*sbderr,*rphase,*phserr,*probf;
int  *stokes,*irets,*baseln,*cursou;  
{
  int which_sol; 
  float local_pherr;
  if(*stokes < 1){*irets=-1; return (-1);} /*no solution available*/
  which_sol=-1; /*which file (i.e. stokes) wanted*/
  if (*stokes == mypol0)which_sol=0; /*means this stokes matches first fringe file*/
  if (*stokes == mypol1)which_sol=1; if (*stokes == mypol2)which_sol=2;if (*stokes == mypol3)which_sol=3;
  if(which_sol <0){*irets=-1; return (-1);} /*no solution available*/
  *baseln=kba1; baseln++; 
  *baseln=kba2;
  *cursou=sourcenum+1;
  *tuts  =scan_reftime/86400.0;
  *intts =scan_stop - scan_start; /*assume this is integration time*/
  if(*intts < 0){*intts =scan_stop - scan_start +86400.0;}  /*midnight*/
  *vamp  = amplitude[which_sol];    /* Coherent amplitude (corr. coeff.) */
  *samp  = inc_seg_ampl[which_sol]*1.0E-4; /* Incoherent segment addition amp.->corrcoeff */
  *rmbd  = resid_mbd[which_sol]*1.0E-6;    /* MBD residual to model (usec->sec) */
  *mbderr= mbd_error[which_sol]*1.0E-6;    /* MBD error calc'd from data (usec->sec) */
  *mbdamb= ambiguity[which_sol]*1.0E-6;    /* MBD ambiguity (usec->sec) */
  *rrate = resid_rate[which_sol]*fourfit_ref_freq; /* Rate residual to model (usec/sec)->Hz */
  *rterr = rate_error[which_sol]*fourfit_ref_freq; /* Rate error calc'd from data (usec/sec)->Hz */
  *rsbd  = resid_sbd[which_sol]*1.0E-6;    /* SBD residual to model (usec) */
  *sbderr= sbd_error[which_sol]*1.0E-6;    /* SBD error calc'd from data (usec) */
  *rphase= resphase[which_sol];     /* Resid phase (deg) */
/* following somewhat empirical from a faulty fourfit*/
  local_pherr= 180.0/(3.14*frg_snr[which_sol]);
  if(fabs(local_pherr) >180)local_pherr=180.0;
  *phserr=local_pherr;
  *probf = prob_false[which_sol];             /* Probability of false detection */
  *irets=0;
  return(0);
}
/*======================================================================================*/
int prej2(first_year,first_day,rae,dece,radate,decdate,gstm)
int first_year,first_day;
double rae,dece;
double *radate,*decdate,*gstm;
{
  int iatoff,iyear;
  double tfl;
  double raepoch,decepoch,rarad,decrad,sfl;
  double dpsi,deps,fract,fint;
  double rap,decp;
  double twopi=6.28318530717959;
  double abk=9.924e-5;
  double abell=1.653e-6;
  double sa,ca,sd,cd,se,ce,td,sl,cl;
  double epsdat,sunlon;
  double rao,deco; 
  double mygast0,my_iat0_epoch;
  double l,lp,o,o2,f2,d2,p,e; 
  double convrd=4.8481368111e-6;
  double zfl,crz,srz,tt2,qfl,xfl,arg,theta,zeta0,z;
  iatoff=32;
  
  iyear= first_year - 1900;
  tfl=(iyear-1)/4+first_day+iyear*365.0-0.5;
  tfl=tfl/36525.0;   /*centuries after 1900 */
  tfl=tfl-1.0; /*centuries after 2000*/
/* nutation stuff */
  l= (485866.733e0 +(1325*1.296e6+ 715922.633e0+31.310e0*tfl)*tfl)*convrd;
  lp=(1287099.804e0+(  99*1.296e6+1292581.224e0 -0.577e0*tfl)*tfl)*convrd;
  f2=(335778.877e0 +(1342*1.296e6+ 295263.137e0-13.257e0*tfl)*tfl)*convrd *2.0;
  d2=(1072261.307e0+(1236*1.296e6+1105601.328e0 -6.891e0*tfl)*tfl)*convrd *2.0;
  o =(450160.280e0 +(  -5*1.296e6- 482890.539e0+ 7.455e0*tfl)*tfl)*convrd;
  o2=o*2.0; 
/*C     NOW EVALUATE SERIES */ 
  p=(-17.2000-1.7e-2*tfl)*sin(o); e=9.2030*cos(o);
  p=p-1.3180*sin(f2-d2+o2);      e=e+0.5740*cos(f2-d2+o2);
  p=p+0.2060*sin(o2);            e=e-0.0900*cos(o2);
  p=p-0.2270*sin(f2+o2);         e=e+0.0980*cos(f2+o2);
  p=p+0.1430*sin(lp); 
  p=p+0.0710*sin(l);
  p=p-0.0520*sin(lp+f2-d2+o2);   e=e+0.0220*cos(lp+f2-d2+o2);
  p=p-0.0390*sin(f2+o);          e=e+0.0200*cos(f2+o);
  p=p-0.0300*sin(l+f2+o2);       e=e+0.0130*cos(l+f2+o2);
  p=p+0.0220*sin(-lp+f2-d2+o2);  e=e-0.0100*cos(-lp+f2-d2+o2);
  p=p+0.0130*sin(f2-d2+o);       e=e-0.0070*cos(f2-d2+o);
  p=p-0.0160*sin(l-d2);
  p=p+0.0120*sin(-l+f2+o2);
  dpsi=p*convrd;
  deps=e*convrd;
/*precession*/
  zeta0=tfl *(1.1180861e-2+1.464e-6*tfl); 
  zfl=  tfl *(1.1180861e-2+5.307e-6*tfl); 
  theta=tfl *(9.7171735e-3-2.068e-6*tfl); 
  crz= cos(rae +zeta0); 
  srz= sin(rae +zeta0);
  tt2= tan(theta/2.0);
  qfl= sin(theta) * (tan(dece)+tt2*crz);
  xfl= atan(qfl * srz / (1.0 - qfl * crz));
  rap= (rae + zeta0 + zfl + xfl);
  arg= tt2 * (crz - srz * tan(xfl / 2.0));
  decp =(dece + 2.0 * atan(arg));
  sa=sin(rap); ca=cos(rap);
  sd=sin(decp); cd=cos(decp);
  td=sd/cd;
  epsdat=0.40909280-2.26966e-4*tfl+deps;
  se=sin(epsdat); ce=cos(epsdat);
  sunlon=4.894950+628.331990*tfl+0.033502*sin(6.240+628.30197*tfl);
  sl=sin(sunlon); cl=cos(sunlon);
  my_iat0_epoch=tfl+iatoff/(86400.0*365.25); /*epoch of iat 0 of first day*/
  my_iat0_epoch=tfl; /*epoch of iat 0 of first day*/
  mygast0=1.75336856+my_iat0_epoch*(628.3319705+my_iat0_epoch*6.77071e-6);
  mygast0=mygast0+deps*ce;
  mygast0=mygast0+2.0*3.1415926535;
  mygast0=mygast0/(2.0*3.1415926535);  /*to turns*/
  fract=modf(mygast0,&fint);  
  mygast0=fract*2.0*3.1415926535; /*back to radians*/
  rao=rap+(ce+se*sa*td)*dpsi-ca*td*deps;
  deco=decp+se*ca*dpsi+sa*deps;
  rap=rao-(9.924e-5*(sl*sa+ce*cl*ca)+1.653e-6*sin(rao+2.945))/cd;
  decp=deco-9.924e-5*(sl*ca*sd-ce*cl*sa*sd+se*cl*cd)
     -1.653e-6*cos(rao+2.945)*sd-1.41e-7*cd;
 *radate=rap; *decdate=decp; *gstm=mygast0;
 return(0);
}
/*------------------precession etc 1950 not yet correct-------------*/
int pre50(first_year,first_day,rae,dece,radate,decdate,gstm)
int first_year,first_day;
double rae,dece;
double *radate,*decdate,*gstm;
{
  int iatoff,iyear;
  double tfl,tday;
  double raepoch,decepoch,rarad,decrad,sfl;
  double dpsi,deps,fract,fint;
  double rap,decp;
  double twopi=6.28318530717959;
  double abk=9.924e-5;
  double abell=1.653e-6;
  double sa,ca,sd,cd,se,ce,td,sl,cl;
  double epsdat,sunlon;
  double rao,deco; 
  double mygast0,my_iat0_epoch;
  double l,lp,o,o2,f2,d2,p,e; 
  double convrd=4.8481368111e-6;
  double zfl,crz,srz,tt2,qfl,xfl,arg,theta,zeta0,z;
  double ela,elb;
  iatoff=32;
/*=============================================*/
/* for 1950, sources spec usually including elliptic aber*/
  ela=0.3410*sin(rae+2.9450)/sin(dece)*1.07;
  elb=(0.3410*cos(rae+2.9450)*sin(dece)+0.0290*cos(dece))*0.94;
  rae=rae+ela*3.1415926535/(180.0*3600.0);
  dece=dece+elb*3.1415926535/(180.0*3600.0);
/*======================================*/
  iyear= first_year - 1900;
  tfl=(iyear-1)/4+first_day+iyear*365.0-0.5;
  tday=tfl; /*sometimes want that in days*/
  tfl =(tfl-1.8262423e4)/36524.21988; /*ephem centuries after 1950*/

/* nutation stuff */
  l=5.1680000+(0.2280271350+1.203e-13*tday)*tday;
  lp=6.2565840+1.720196977e-2*tday;
  f2=0.3927300+(0.46179144660-8.402e-14*tday)*tday;
  d2=5.9598630+(0.42553742330-3.756e-14*tday)*tday;
  o=4.5236020-(9.2422029e-4-2.717e-14*tday)*tday;
  o2=o*2.0; 
/*C     NOW EVALUATE SERIES */
  p=(-17.189-4.7e-7*tfl)*sin(o); e=9.2030*cos(o);
  p=p-1.3140*sin(f2-d2+o2);      e=e+0.5720*cos(f2-d2+o2);
  p=p+0.2090*sin(o2);            e=e-0.0900*cos(o2);
  p=p-0.2040*sin(f2+o2);         e=e+0.0880*cos(f2+o2);
  p=p+0.1450*sin(lp); 
  p=p+0.0680*sin(l);
  p=p-0.0500*sin(lp+f2-d2+o2);   e=e+0.0220*cos(lp+f2-d2+o2);
  p=p-0.0340*sin(f2+o);          e=e+0.0180*cos(f2+o);
  p=p-0.0260*sin(l+f2+o2);       e=e+0.0110*cos(l+f2+o2);
  p=p+0.0210*sin(-lp+f2-d2+o2);  
  p=p+0.0120*sin(f2-d2+o);      
  p=p-0.0150*sin(l-d2);
  p=p+0.0110*sin(-l+f2+o2);
  dpsi=p*convrd;
  deps=e*convrd;
/*precession*/
  zeta0=tfl *(1.1177209e-2+1.464e-6*tfl); 
  zfl=  tfl *(1.1177209e-2+5.299e-6*tfl); 
  theta=tfl *(9.7190792e-3-2.065e-6*tfl); 
  crz= cos(rae +zeta0); 
  srz= sin(rae +zeta0);
  tt2= tan(theta/2.0);
  qfl= sin(theta) * (tan(dece)+tt2*crz);
  xfl= atan(qfl * srz / (1.0 - qfl * crz));
  rap= (rae + zeta0 + zfl + xfl);
  arg= tt2 * (crz - srz * tan(xfl / 2.0));
  decp =(dece + 2.0 * atan(arg));
  sa=sin(rap); ca=cos(rap);
  sd=sin(decp); cd=cos(decp);
  td=sd/cd;
  epsdat=0.4093197552-6.217959450e-9*tday+deps;
  se=sin(epsdat); ce=cos(epsdat);
  sunlon=4.881628+1.720279127e-2*tday+0.033502*sin(6.2565835+1.720196977e-2*tday);
  sl=sin(sunlon); cl=cos(sunlon);
  my_iat0_epoch=tday+iatoff/(86400.0); /*epoch of iat 0 of first day*/
  mygast0=1.73993589+tday*(1.7202791266e-2+tday*5.064e-15);
  mygast0=mygast0+deps*ce;
  mygast0=mygast0+2.0*3.1415926535;
  mygast0=mygast0/(2.0*3.1415926535);  /*to turns*/
  fract=modf(mygast0,&fint);  
  mygast0=fract*2.0*3.1415926535; /*back to radians*/
  rao=rap+(ce+se*sa*td)*dpsi-ca*td*deps;
  deco=decp+se*ca*dpsi+sa*deps;
  rap=rao-(9.924e-5*(sl*sa+ce*cl*ca)+1.653e-6*sin(rao+2.945))/cd;
  decp=deco-9.924e-5*(sl*ca*sd-ce*cl*sa*sd+se*cl*cd)
     -1.653e-6*cos(rao+2.945)*sd-1.41e-7*cd;
 *radate=rap; *decdate=decp; *gstm=mygast0;
 return(0);
}

